import std/[json, options, strutils]
from logging import Level
import ../namespaced_logging

export
  # Types
  Level,
  Logger,
  LogAppender,
  LogMessage,
  ConsoleLogAppender,
  CustomLogAppender,
  CustomLogAppenderFunc,
  FileLogAppender,

  # Procs/Funcs
  `%`,
  initConsoleLogAppender,
  initCustomLogAppender,
  initFileLogAppender,
  formatJsonStructuredLog,
  useForAutoconfiguredLogging


proc setRootLoggingThreshold*(lvl: Level) =
  setRootThreshold(getAutoconfiguredLogService(), lvl)


proc setLoggingThreshold*(scope: string, lvl: Level) =
  setThreshold(getAutoconfiguredLogService(), scope, lvl)


proc addLogAppender*(appender: LogAppender) =
  addAppender(getAutoconfiguredLogService(), appender)


proc clearLogAppenders*() =
  clearAppenders(getAutoconfiguredLogService())


proc getLogger*(scope: string, lvl: Option[Level] = none[Level]()): Logger =
  getLogger(getAutoconfiguredLogService(), scope, lvl)


template log*(lm: LogMessage) = log(getAutoconfiguredLogger(), lm)

template log*(lvl: Level, msg: untyped) = log(getAutoconfiguredLogger(), lvl, msg)

template log*[T: ref Exception](lvl: Level, error: T, msg: untyped) =
  log(getAutoconfiguredLogger(), lvl, error, msg)

template debug*[T](msg: T) = log(lvlDebug, msg)
template info*[T](msg: T) = log(lvlInfo, msg)
template notice*[T](msg: T) = log(lvlNotice, msg)
template warn*[T](msg: T) = log(lvlWarn, msg)
template error*[T](msg: T) = log(lvlError, msg)
template error*[T](error: ref Exception, msg: T) = log(lvlError, error, msg)
template fatal*[T](msg: T) = log(lvlFatal, msg)
template fatal*[T](error: ref Exception, msg: T) = log(lvlFatal, error, msg)

when isMainModule:
  import std/unittest
  import ./testutil

  suite "Autoconfigured Logging":
    setup:
      globalLogServiceRef[] = initLogService()
      let loggedMessages = initLoggedMessages()
      let testAppender = initTestLogAppender(loggedMessages)

    test "simple no-config logging":
      addLogAppender(testAppender)
      info("test message")

      let lm = loggedMessages.get()
      check:
        lm.len == 1
        lm[0].level == lvlInfo
        lm[0].message == "test message"
